from unittest import TestCase
from unittest.mock import MagicMock

from awehflow.events.alerts import AlertsEventHandler


class TestAlertsEventHandler(TestCase):
    def test_handle_not_alerting(self):
        alerter1 = MagicMock()
        alerter2 = MagicMock()
        handler = AlertsEventHandler(alerters=[alerter1, alerter2], alert_on=['success'])

        event = {'name': 'woot'}
        handler.handle(event)

        alerter1.alert.assert_not_called()
        alerter2.alert.assert_not_called()

    def test_handle_alerting(self):
        alerter1 = MagicMock()
        alerter2 = MagicMock()
        handler = AlertsEventHandler(alerters=[alerter1, alerter2], alert_on=['success'])

        event = {'name': 'success'}
        handler.handle(event)

        alerter1.alert.assert_called_once_with(event)
        alerter2.alert.assert_called_once_with(event)



import datetime
from unittest import TestCase

import pendulum

from awehflow.config import Config


class TestConfig(TestCase):

    def setUp(self):
        self.dummy_config_dict = {
            'name': 'some_name',
            'version': 2,
            'start_date': '2020-03-29',
            'schedule': '30 23 * * *',
            'catchup': True,
            'timezone': 'Africa/Johannesburg',
            'engineers': [
                {
                    'name': 'John Doe',
                    'slack': 'johndoe'
                }
            ],
            'params': {
                'default': {
                    'var1': 'var1 value',
                    'var2': 'var2 value'
                },
                'prod': {
                    'var2': 'var2 prod value'
                }
            }
        }

    def test_config_init(self):
        config = Config(self.dummy_config_dict)

        self.assertEqual(config.get('name'), 'some_name')
        self.assertEqual(config.get('dag_id'), 'some_name_v2')
        self.assertEqual(config.get('start_date'),
                         datetime.datetime(2020, 3, 29, tzinfo=pendulum.timezone('Africa/Johannesburg')))
        self.assertEqual(config.get('schedule'), '30 23 * * *')
        self.assertEqual(config.get('engineers.0.name'), 'John Doe')
        self.assertEqual(config.get('engineers.0.slack'), 'johndoe')
        self.assertEqual(config.get('params.var1'), 'var1 value')
        self.assertEqual(config.get('params.var2'), 'var2 value')
        self.assertEqual(config.get('catchup'), True)
        self.assertEqual(config.get('default_dag_args'), {})

    def test_variables_prod(self):
        config = Config(self.dummy_config_dict, environment='prod')

        self.assertEqual(config.get('params.var1'), 'var1 value')
        self.assertEqual(config.get('params.var2'), 'var2 prod value')




from unittest import TestCase
from unittest.mock import patch

from awehflow.utils import merge_dicts, random_string, uniquify_name


class TestUtils(TestCase):

    def test_merge_dicts(self):
        self.assertEqual(merge_dicts({'a': 1}, {'b': 2}), {'a': 1, 'b': 2})

    def test_nested_merge_dicts(self):
        self.assertEqual(merge_dicts({
            'a': 1,
            'c': {
                'd': 5,
                'e': 6
            }
        },
        {
            'b': 2,
            'c': {
                'e': 7
            }
        }), {
            'a': 1,
            'b': 2,
            'c': {
                'd': 5,
                'e': 7
            }
        })

    def test_random_string(self):
        str1 = random_string(string_length=10)
        self.assertEqual(len(str1), 10)

        str2 = random_string(string_length=20)
        self.assertEqual(len(str2), 20)

        str3 = random_string(string_length=20)
        self.assertNotEqual(str2, str3)

    @patch('awehflow.utils.random_string', return_value='bla_bla_bla')
    def test_uniquify_name(self, random_string_mock):
        self.assertEqual(uniquify_name('some_name'), 'some_name_bla_bla_bla')



from unittest import TestCase
from unittest.mock import patch

from awehflow.alerts.slack import SlackAlerter


class TestSlackAlerter(TestCase):
    def setUp(self):
        self.alerter = SlackAlerter(channel='#channel_name')

    @patch('awehflow.alerts.slack.slack')
    @patch('awehflow.alerts.slack.BaseHook')
    def test_success_alert(self, MockBaseHook, mock_slack):
        MockBaseHook.get_connection.return_value.password = 'token'

        client = mock_slack.WebClient.return_value

        self.alerter.alert({
            'name': 'success',
            'body': {
                'key': 'value',
                'key2': 'value2',
                'project': 'some_project'
            }
        })

        client.chat_postMessage.assert_called_once_with(
            channel='#channel_name',
            text='SUCCESS',
            blocks=[
                {
                    "type": "section",
                    "text": {
                        "type": "mrkdwn",
                        "text": ":white_check_mark: *SUCCESS*"
                    }
                },
                {
                    "type": "divider"
                },
                {
                    "type": "section",
                    "fields": [
                        {
                            "type": "mrkdwn",
                            "text": "*key*\nvalue"
                        },
                        {
                            "type": "mrkdwn",
                            "text": "*key2*\nvalue2"
                        }
                    ]
                },
                {
                    "type": "context",
                    "elements": [
                        {
                            "type": "mrkdwn",
                            "text": ":pushpin: *some_project*"
                        }
                    ]
                }
            ]
        )

    @patch('awehflow.alerts.slack.slack')
    @patch('awehflow.alerts.slack.BaseHook')
    def test_failure_alert(self, MockBaseHook, mock_slack):
        MockBaseHook.get_connection.return_value.password = 'token'

        client = mock_slack.WebClient.return_value

        self.alerter.alert({
            'name': 'failure',
            'body': {
                'name': 'dag_name',
                'key': 'value',
                'key2': 'value2',
                'project': 'some_project',
                'error': {
                    'task_id': 'some_task_id',
                    'message': 'error message',
                    'log_url': 'example.com'
                },
                'engineers': [
                    {
                        'slack': 'piet'
                    },
                    {
                        'slack': 'pompies'
                    },
                    {
                        'email': 'piet@pompies.com'
                    }
                ]
            }
        })

        client.chat_postMessage.assert_called_once_with(
            channel='#channel_name',
            text='FAILURE',
            blocks=[
                {
                    "type": "section",
                    "text": {
                        "type": "mrkdwn",
                        "text": ":rotating_light: *<example.com|FAILURE>*"
                    }
                },
                {
                    "type": "divider"
                },
                {
                    "type": "section",
                    "fields": [
                        {
                            "type": "mrkdwn",
                            "text": "*DAG*\ndag_name"
                        },
                        {
                            "type": "mrkdwn",
                            "text": "*Task*\nsome_task_id"
                        },
                        {
                            "type": "mrkdwn",
                            "text": "*Exception*\nerror message"
                        },
                        {
                            "type": "mrkdwn",
                            "text": "*Engineers*\n<@piet>, <@pompies>"
                        }
                    ]
                },
                {
                    "type": "context",
                    "elements": [
                        {
                            "type": "mrkdwn",
                            "text": ":pushpin: *some_project*"
                        }
                    ]
                }
            ]
        )


from airflow.models import BaseOperator
from airflow.utils.decorators import apply_defaults

from awehflow.utils import utc_now, JobStatus


class EventEmittingOperator(BaseOperator):
    @apply_defaults
    def __init__(
            self,
            event_handlers=None,
            *args, **kwargs):
        """
        :param event_handlers: List of event handlers to emit events to
        """

        super(EventEmittingOperator, self).__init__(*args, **kwargs)
        if event_handlers is None:
            event_handlers = []
        self.event_handlers = event_handlers

    def execute(self, context):
        raise Exception("EventEmittingOperator should be extended")

    def emit_event(self, event_name, body={}):
        for handler in self.event_handlers:
            handler.handle({
                'name': event_name,
                'body': body
            })


class FlowOperator(EventEmittingOperator):
    @apply_defaults
    def __init__(
            self,
            project,
            job_name,
            engineers,
            *args, **kwargs):
        """
        :param job_name: Name of the pipeline job. Typically just use the dag_name
        :param engineers: list of engineers responsible for the pipeline
        """
        super(FlowOperator, self).__init__(*args, **kwargs)
        self.project = project
        self.job_name = job_name
        self.engineers = engineers


class StartOperator(FlowOperator):
    def execute(self, context):
        self.emit_event('start', {
            'run_id': context['dag_run'].run_id,
            'dag_id': self.dag.dag_id,
            'name': self.job_name,
            'project': self.project,
            'engineers': self.engineers,
            'status': JobStatus.RUNNING,
            'start_time': utc_now(),
            'reference_time': context['next_execution_date']
        })


class SuccessOperator(FlowOperator):
    def execute(self, context):
        self.emit_event('success', {
            'run_id': context['dag_run'].run_id,
            'dag_id': self.dag.dag_id,
            'name': self.job_name,
            'project': self.project,
            'engineers': self.engineers,
            'status': JobStatus.SUCCESS,
            'end_time': utc_now()
        })


class FailureOperator(FlowOperator):
    def execute(self, context):
        self.emit_event('failure', {
            'run_id': context['dag_run'].run_id,
            'dag_id': self.dag.dag_id,
            'name': self.job_name,
            'project': self.project,
            'engineers': self.engineers,
            'status': JobStatus.FAILURE,
            'error': {
                'task_id': context['task_instance'].task_id,
                'message': str(context.get('exception', '')),
                'log_url': context['task_instance'].log_url
            },
            'end_time': utc_now()
        })


import datetime
from unittest import TestCase
from unittest.mock import MagicMock, patch, call

import pytz
from airflow import DAG
from airflow.models import TaskInstance

from awehflow.operators.gcp import BigQueryJobOperator, BigQueryJobTaskMetricOperator, BigQueryShortCircuitOperator


class TestBigQueryJobOperator(TestCase):
    def setUp(self):
        self.dag = DAG(dag_id='some_dag', start_date=datetime.datetime.now())

        self.task = BigQueryJobOperator(
            dag=self.dag,
            task_id='task',
            sql='select 1;'
        )

        self.task_instance = TaskInstance(task=self.task, execution_date=datetime.datetime(1985, 3, 29))
        self.task_instance_context = self.task_instance.get_template_context()

    @patch('awehflow.operators.gcp.BigQueryHook')
    def test_execute(self, _):
        self.task.bq_cursor = MagicMock()
        self.task.bq_cursor.run_query.return_value = 'some_job_id'
        job_id = self.task.execute(self.task_instance_context)
        self.assertEqual(job_id, 'some_job_id')


class TestBigQueryJobTaskMetricOperator(TestCase):
    def setUp(self):
        self.dag = DAG(dag_id='some_dag', start_date=datetime.datetime.now())

        self.task = BigQueryJobTaskMetricOperator(
            dag=self.dag,
            task_id='task',
            task_ids=['task_id_1', 'task_id_2'],
            params={
                'job_name': 'some_job_name'
            }
        )

        self.task_instance = TaskInstance(task=self.task, execution_date=datetime.datetime(1985, 3, 29, tzinfo=pytz.UTC))
        self.task_instance_context = self.task_instance.get_template_context()

    @patch('awehflow.operators.gcp.BigQueryHook')
    @patch('awehflow.operators.gcp.utc_now')
    def test_execute(self, mock_utc_now, MockBigQueryHook):
        self.task.emit_event = MagicMock()
        mock_utc_now.return_value = datetime.datetime(1985, 3, 29)
        jobs = MockBigQueryHook.return_value.get_service.return_value.jobs.return_value
        jobs.get.return_value.execute.return_value = 'some_metric'
        ti = MagicMock()
        ti.xcom_pull.return_value = 'some_job_id'
        self.task_instance_context['task_instance'] = ti
        self.task_instance_context['dag_run'] = MagicMock()
        self.task_instance_context['dag_run'].run_id = 'some_dag_run_id'

        self.task.execute(self.task_instance_context)

        self.task.emit_event.assert_has_calls([
            call('task_metric', {
                'run_id': 'some_dag_run_id',
                'dag_id': 'some_dag',
                'job_name': 'some_job_name',
                'task_id': 'task_id_1',
                'value': 'some_metric',
                'created_time': datetime.datetime(1985, 3, 29),
                'reference_time': datetime.datetime(1985, 3, 30, tzinfo=pytz.UTC)
            }),
            call('task_metric', {
                'run_id': 'some_dag_run_id',
                'dag_id': 'some_dag',
                'job_name': 'some_job_name',
                'task_id': 'task_id_2',
                'value': 'some_metric',
                'created_time': datetime.datetime(1985, 3, 29),
                'reference_time': datetime.datetime(1985, 3, 30, tzinfo=pytz.UTC)
            })
        ], any_order=True)


class TestBigQueryShortCircuitOperator(TestCase):
    def setUp(self):
        self.dag = DAG(dag_id='some_dag', start_date=datetime.datetime.now())

        self.task = BigQueryShortCircuitOperator(
            dag=self.dag,
            task_id='task',
            sql="SELECT 1;"
        )

        self.task_instance = TaskInstance(task=self.task, execution_date=datetime.datetime(1985, 3, 29, tzinfo=pytz.UTC))
        self.task_instance_context = self.task_instance.get_template_context()

    @patch('awehflow.operators.gcp.BigQueryHook')
    def test_execute(self, MockBigQueryHook):

        self.task_instance_context['task'] = MagicMock()
        self.task_instance_context['task'].get_flat_relatives.return_value = ['task']

        self.task.skip = MagicMock()
        MockBigQueryHook.return_value.get_first.return_value = [True]
        self.task.execute(self.task_instance_context)
        self.task.skip.assert_not_called()

        self.task.skip = MagicMock()
        MockBigQueryHook.return_value.get_first.return_value = [True, 1, 'something']
        self.task.execute(self.task_instance_context)
        self.task.skip.assert_not_called()

        self.task.skip = MagicMock()
        MockBigQueryHook.return_value.get_first.return_value = [False]
        self.task.execute(self.task_instance_context)
        self.task.skip.assert_called_once()

        self.task.skip = MagicMock()
        MockBigQueryHook.return_value.get_first.return_value = [0, '', None]
        self.task.execute(self.task_instance_context)
        self.task.skip.assert_called_once()

        self.task.skip = MagicMock()
        MockBigQueryHook.return_value.get_first.return_value = None
        self.task.execute(self.task_instance_context)
        self.task.skip.assert_called_once()




import datetime
from unittest import TestCase
from unittest.mock import patch, MagicMock

from awehflow.events.postgres import PostgresMetricsEventHandler


class TestPostgresMetricsEventHandler(TestCase):
    @patch('awehflow.events.postgres.PostgresHook')
    def setUp(self, _):
        self.handler = PostgresMetricsEventHandler(
            jobs_table='some_jobs_table',
            task_metrics_table='some_task_metrics_table'
        )

    def test_handle(self):
        """All events should literally just pass the event the event along to pubsub"""
        self.handler._PostgresMetricsEventHandler__upsert_job = MagicMock()
        event = {'name': 'start'}
        self.handler.handle(event)
        self.handler._PostgresMetricsEventHandler__upsert_job.assert_called_once_with(event)

        self.handler._PostgresMetricsEventHandler__upsert_job = MagicMock()
        event = {'name': 'success'}
        self.handler.handle(event)
        self.handler._PostgresMetricsEventHandler__upsert_job.assert_called_once_with(event)

        self.handler._PostgresMetricsEventHandler__upsert_job = MagicMock()
        event = {'name': 'failure'}
        self.handler.handle(event)
        self.handler._PostgresMetricsEventHandler__upsert_job.assert_called_once_with(event)

        self.handler._PostgresMetricsEventHandler__insert_task_metric = MagicMock()
        event = {'name': 'task_metric'}
        self.handler.handle(event)
        self.handler._PostgresMetricsEventHandler__insert_task_metric.assert_called_once_with(event)

        self.handler._PostgresMetricsEventHandler__upsert_job = MagicMock()
        event = {'name': 'woot'}
        self.handler.handle(event)
        self.handler._PostgresMetricsEventHandler__upsert_job.assert_not_called()

    def test_upsert_job(self):
        self.handler.hook = MagicMock()

        self.handler._PostgresMetricsEventHandler__upsert_job({
            'name': 'something',
            'body': {
                'run_id': 'some_run_id',
                'dag_id': 'some_dag_id',
                'name': 'some_name',
                'engineers': [
                    {
                        'name': 'Batman'
                    }
                ],
                'start_time': datetime.datetime(1985, 3, 29),
                'reference_time': datetime.datetime(1986, 3, 29)
            }
        })
        self.handler.hook.run.assert_called_once_with(
            sql="""
            INSERT INTO some_jobs_table (run_id,dag_id,name,engineers,start_time,reference_time)
            VALUES (%s,%s,%s,%s,%s,%s)
            ON CONFLICT ON CONSTRAINT run_id_dag_id_unique DO UPDATE SET (name,engineers,start_time,reference_time) = (EXCLUDED.name,EXCLUDED.engineers,EXCLUDED.start_time,EXCLUDED.reference_time)
        """,
            parameters=('some_run_id', 'some_dag_id', 'some_name', '[{"name": "Batman"}]', '1985-03-29 00:00:00', '1986-03-29 00:00:00')
        )

    def test_insert_task_metric(self):
        self.handler.hook = MagicMock()

        self.handler._PostgresMetricsEventHandler__insert_task_metric({
            'name': 'something',
            'body': {
                'run_id': 'some_run_id',
                'dag_id': 'some_dag_id',
                'job_name': 'some_job_name',
                'task_id': 'some_task_id',
                'value': {
                    'name': 'Batman'
                },
                'created_time': datetime.datetime(1985, 3, 29),
                'reference_time': datetime.datetime(1986, 3, 29)
            }
        })
        self.handler.hook.run.assert_called_once_with(
            sql="""
            INSERT INTO some_task_metrics_table (run_id,dag_id,job_name,task_id,value,created_time,reference_time)
            VALUES (%s,%s,%s,%s,%s,%s,%s)
        """,
            parameters=('some_run_id', 'some_dag_id', 'some_job_name', 'some_task_id', '{"name": "Batman"}', '1985-03-29 00:00:00', '1986-03-29 00:00:00')
        )


from unittest import TestCase
from unittest.mock import patch, MagicMock

from awehflow.events.gcp import PublishToGooglePubSubEventHandler


class TestPublishToGooglePubSubEventHandler(TestCase):
    @patch('awehflow.events.gcp.PubSubHook')
    def setUp(self, _):
        self.handler = PublishToGooglePubSubEventHandler(
            project_id='some_project_id',
            topic='some_topic'
        )

    def test_handle(self):
        """All events should literally just pass the event the event along to pubsub"""
        self.handler._PublishToGooglePubSubEventHandler__emit_message = MagicMock()
        event = {'name': 'woot'}
        self.handler.handle(event)
        self.handler._PublishToGooglePubSubEventHandler__emit_message.assert_called_once_with(event)

        self.handler._PublishToGooglePubSubEventHandler__emit_message = MagicMock()
        event = {'name': 'start'}
        self.handler.handle(event)
        self.handler._PublishToGooglePubSubEventHandler__emit_message.assert_called_once_with(event)

        self.handler._PublishToGooglePubSubEventHandler__emit_message = MagicMock()
        event = {'name': 'failure'}
        self.handler.handle(event)
        self.handler._PublishToGooglePubSubEventHandler__emit_message.assert_called_once_with(event)

    def test_emit_message(self):
        self.handler._PublishToGooglePubSubEventHandler__emit_batch = MagicMock()
        self.handler._PublishToGooglePubSubEventHandler__emit_message({})
        self.handler._PublishToGooglePubSubEventHandler__emit_batch.assert_called_once()

    def test_emit_batch(self):
        self.handler._PublishToGooglePubSubEventHandler__emit_batch([{'msg': 'hello world'}])
        self.handler.pubsub_hook.publish.assert_called_once_with('some_project_id', 'some_topic', [{'msg': 'hello world'}])

    def test_encode_message(self):
        encoded_message = PublishToGooglePubSubEventHandler._PublishToGooglePubSubEventHandler__encode_message('Hello World!')
        self.assertEqual(encoded_message, 'IkhlbGxvIFdvcmxkISI=')


from json import dumps
from httplib2 import Http
from airflow.hooks.base_hook import BaseHook

from awehflow.alerts.base import Alerter


class GoogleChatAlerter(Alerter):
    def __init__(self, gchat_conn_id='gchat_default'):
        self.gchat_conn_id = gchat_conn_id

    def alert(self, context):
        client = Http()
        message_header = {'Content-Type': 'application/json; charset=UTF-8'}
        uri = BaseHook.get_connection(self.gchat_conn_id).password

        client.request(
            uri=uri,
            method='POST',
            headers=message_header,
            body=dumps(self.__generate_blocks(context)),
        )

    def __message_title(self, context):
        return context.get('name', 'alert').upper()

    def __message_header(self, context):
        if context.get('name', '') == 'success':
            return "*{}*".format(self.__message_title(context))
        else:
            return "*{} <users/all>*".format(self.__message_title(context))

    def __generate_message_body(self, items_dict):
        return {
            "widgets": [
                {
                    "keyValue": {
                        "topLabel": "{}".format(key),
                        "content": "{}".format(value),
                    },
                } for key, value in items_dict.items()]
        }

    def __message_body(self, context):
        if context['name'] == 'failure':
            items = {
                'DAG': context['body']['name'],
                'Task': context['body']['error']['task_id'],
                'Exception': context['body']['error']['message'],
            }
            if 'engineers' in context['body']:
                engineer_name = []
                for engineer in context['body']['engineers']:
                    if 'name' in engineer:
                        engineer_name.append(engineer['name'])
                if engineer_name:
                    items['Engineers'] = ', '.join(['{}'.format(engineer) for engineer in engineer_name])
        else:
            items = {key: context['body'][key] for key in context['body'].keys() if
                     key not in ['project', 'engineers', 'status']}

        return self.__generate_message_body(items)

    def __message_footer(self, context):
        return {
            "keyValue": {
                "topLabel": "Project",
                "content": "{}".format(context['body']['project'])
            }
        }

    def __message_url(self, context):
        if context['name'] == 'failure':
            return {
               "buttons": [
                   {
                       "textButton": {
                           "text": "VIEW LOGS",
                           "onClick": {
                               "openLink": {
                                   "url": "{}".format(context['body']['error']['log_url'])
                               }
                           }
                       }
                   }
               ]
            }
        else:
            return {}

    def __generate_blocks(self, context):
        blocks = {
            "cards": [
                {
                    "sections": [
                        self.__message_body(context),
                        {
                            "widgets": [
                                self.__message_footer(context)
                            ]
                        },
                        {
                            "widgets": [
                                self.__message_url(context),
                            ]
                        },
                    ]
                }
            ],
            "text": self.__message_header(context)
        }

        return blocks



import json
import logging

from airflow.hooks.postgres_hook import PostgresHook

from awehflow.events.base import EventHandler


class PostgresMetricsEventHandler(EventHandler):
    JOBS_SCHEMA = {
        'id': {
            'type': 'serial'
        },
        'run_id': {
            'type': 'varchar'
        },
        'dag_id': {
            'type': 'varchar'
        },
        'name': {
            'type': 'varchar'
        },
        'project': {
            'type': 'varchar'
        },
        'status': {
            'type': 'varchar'
        },
        'engineers': {
            'type': 'json'
        },
        'error': {
            'type': 'json'
        },
        'start_time': {
            'type': 'timestamp'
        },
        'end_time': {
            'type': 'timestamp'
        },
        'reference_time': {
            'type': 'timestamp'
        }
    }

    JOB_TASK_METRICS_SCHEMA = {
        'id': {
            'type': 'serial'
        },
        'run_id': {
            'type': 'varchar'
        },
        'dag_id': {
            'type': 'varchar'
        },
        'job_name': {
            'type': 'varchar'
        },
        'task_id': {
            'type': 'varchar'
        },
        'value': {
            'type': 'json'
        },
        'created_time': {
            'type': 'timestamp'
        },
        'reference_time': {
            'type': 'timestamp'
        }
    }

    def __init__(self, jobs_table, task_metrics_table, postgres_conn_id='postgres_jobs'):
        self.hook = PostgresHook(postgres_conn_id=postgres_conn_id)
        self.jobs_table = jobs_table
        self.task_metrics_table = task_metrics_table

    def start(self, event):
        logging.info('start {}'.format(event))
        self.__upsert_job(event)

    def success(self, event):
        logging.info('success {}'.format(event))
        self.__upsert_job(event)

    def failure(self, event):
        logging.info('failure {}'.format(event))
        self.__upsert_job(event)

    def task_metric(self, event):
        logging.info('task_metric: {}'.format(event))
        self.__insert_task_metric(event)

    @staticmethod
    def __get_value(schema, event_body, key):
        if key not in schema:
            return event_body.get(key, '')
        if schema[key]['type'] == 'json':
            return json.dumps(event_body.get(key, ''))
        if schema[key]['type'] == 'timestamp':
            return str(event_body.get(key, ''))
        return event_body.get(key, schema[key].get('default_value', ''))

    def __upsert_job(self, event):
        body = event.get('body', {})
        columns = body.keys()
        values = [self.__get_value(self.JOBS_SCHEMA, body, column) for column in columns]
        value_placeholders = ['%s' for _ in values]
        update_columns = [column for column in columns if column not in ['run_id', 'dag_id']]
        update_values = ['EXCLUDED.{}'.format(column) for column in update_columns]
        sql = """
            INSERT INTO {table_name} ({columns})
            VALUES ({value_placeholders})
            ON CONFLICT ON CONSTRAINT run_id_dag_id_unique DO UPDATE SET ({update_columns}) = ({update_values})
        """.format(
            table_name=self.jobs_table,
            columns=','.join(columns),
            value_placeholders=','.join(value_placeholders),
            update_columns=','.join(update_columns),
            update_values=','.join(update_values)
        )

        self.hook.run(
            sql=sql,
            parameters=tuple(values)
        )

    def __insert_task_metric(self, event):
        body = event.get('body', {})
        columns = [key for key in self.JOB_TASK_METRICS_SCHEMA.keys() if key != 'id']
        values = [self.__get_value(self.JOB_TASK_METRICS_SCHEMA, body, column) for column in columns]
        value_placeholders = ['%s' for _ in values]
        sql = """
            INSERT INTO {table_name} ({columns})
            VALUES ({value_placeholders})
        """.format(
            table_name=self.task_metrics_table,
            columns=','.join(columns),
            value_placeholders=','.join(value_placeholders)
        )

        self.hook.run(
            sql=sql,
            parameters=tuple(values)
        )


import datetime
import json
import os
from unittest import TestCase

from airflow import DAG
from airflow.models import BaseOperator
from airflow.operators.bash_operator import BashOperator
from airflow.operators.python_operator import PythonOperator

from awehflow.config import Config
from awehflow.core import ConfigCache, DagLoader, SubDag, DependencySensors, TasksSubDag
from tests.helpers import prepare_clean_tmp_dir, tmp_dir_path, ensure_dir_exists, dict_to_yml


class TestConfigCache(TestCase):

    def setUp(self):
        self.configs = [
            {
                'name': 'pipeline_1',
                'start_date': '2020-03-29',
                'schedule': '30 23 * * *',
                'catchup': True,
                'engineers': [
                    {
                        'name': 'Engineer 1',
                        'slack': 'engineer_1'
                    }
                ]
            },
            {
                'name': 'pipeline_2',
                'start_date': '2020-03-30',
                'engineers': [
                    {
                        'name': 'Engineer 2',
                        'slack': 'engineer_2'
                    }
                ]
            }
        ]

        prepare_clean_tmp_dir(folder_name='config_cache')
        self.configs_path = os.path.join(tmp_dir_path(), 'config_cache')
        ensure_dir_exists(self.configs_path)

        for config in self.configs:
            config_path = os.path.join(self.configs_path, '{}.yml'.format(config['name']))
            dict_to_yml(config, config_path)

        self.config_cache = ConfigCache(self.configs_path)

    def test_cache_directory_path(self):
        self.assertEqual(self.config_cache.cache_directory_path, '{}/.cache'.format(self.configs_path))

    def test_bundle_path(self):
        self.assertEqual(self.config_cache.bundle_path, '{}/.cache/configs_bundle.json'.format(self.configs_path))

    def test_config_files_glob(self):
        self.assertEqual(len(self.config_cache.config_files_glob), 2)
        self.assertCountEqual([file.split('/')[-1] for file in self.config_cache.config_files_glob],
                              ['pipeline_1.yml', 'pipeline_2.yml'])

    def test_refresh_cache(self):
        self.config_cache.refresh_cache()
        self.assertTrue(os.path.exists(self.config_cache.bundle_path))

        with open(self.config_cache.bundle_path) as json_file:
            configs_read_from_bundle = json.load(json_file)

        self.assertCountEqual(self.configs, configs_read_from_bundle)

    def test_read_cache(self):
        with self.assertRaises(FileNotFoundError):
            self.config_cache.read_cache()

        self.config_cache.refresh_cache()
        configs = self.config_cache.read_cache(environment='prod')
        self.assertCountEqual([Config(config, environment='prod') for config in self.configs], configs)


class TestDagLoader(TestCase):
    def setUp(self):
        self.configs = [
            {
                'name': 'pipeline_a',
                'start_date': '2020-03-29',
                'schedule': '30 23 * * *',
                'catchup': True,
                'engineers': [
                    {
                        'name': 'Engineer 1',
                        'slack': 'engineer_1'
                    }
                ],
                'default_dag_args': {
                    'foo': 'bar'
                },
                'tasks': [
                    {
                        'id': 'stage_1',
                        'operator': 'airflow.operators.bash_operator.BashOperator',
                        'params': {
                            'bash_command': 'echo "hello world"'
                        }
                    },
                    {
                        'id': 'stage_2',
                        'operator': 'airflow.operators.bash_operator.BashOperator',
                        'params': {
                            'bash_command': 'echo "hello universe"'
                        },
                        'upstream': ['stage_1']
                    },
                    {
                        'id': 'stage_2b',
                        'operator': 'airflow.operators.bash_operator.BashOperator',
                        'params': {
                            'bash_command': 'echo "hello universe in parallel"'
                        },
                        'upstream': ['stage_1']
                    }
                ]
            },
            {
                'name': 'pipeline_b',
                'start_date': '2020-03-30',
                'engineers': [
                    {
                        'name': 'Engineer 2',
                        'slack': 'engineer_2'
                    }
                ],
                'dependencies': [
                    {
                        'id': 'sensor_1',
                        'operator': 'airflow.sensors.time_sensor.TimeSensor',
                        'params': {
                            'target_time': '2020-03-29 00:00:00'
                        }
                    },
                    {
                        'id': 'sensor_2',
                        'operator': 'airflow.sensors.time_sensor.TimeSensor',
                        'params': {
                            'target_time': '2020-03-29 00:00:00'
                        }
                    },
                ],
                'tasks': [
                    {
                        'id': 'stage_1b',
                        'operator': 'airflow.operators.bash_operator.BashOperator',
                        'params': {
                            'bash_command': 'echo "hello world"'
                        }
                    },
                    {
                        'id': 'stage_2c',
                        'operator': 'airflow.operators.bash_operator.BashOperator',
                        'params': {
                            'bash_command': 'echo "hello world"'
                        }
                    }
                ]
            },
            {
                'name': 'pipeline_c',
                'start_date': '2020-03-30',
                'engineers': [
                    {
                        'name': 'Engineer 3',
                        'slack': 'engineer_3'
                    }
                ],
                'pre_hooks': [
                    {
                        'id': 'pre_hook_1',
                        'operator': 'airflow.sensors.time_sensor.TimeSensor',
                        'params': {
                            'target_time': '2020-03-29 00:00:00'
                        }
                    },
                    {
                        'id': 'pre_hook_2',
                        'operator': 'airflow.sensors.time_sensor.TimeSensor',
                        'params': {
                            'target_time': '2020-03-29 00:00:00'
                        },
                        'upstream': ['pre_hook_1']
                    }
                ],
                'dependencies': [
                    {
                        'id': 'sensor_3',
                        'operator': 'airflow.sensors.time_sensor.TimeSensor',
                        'params': {
                            'target_time': '2020-03-29 00:00:00'
                        }
                    }
                ],
                'tasks': [
                    {
                        'id': 'stage_3',
                        'operator': 'airflow.operators.bash_operator.BashOperator',
                        'params': {
                            'bash_command': 'echo "hello world"'
                        }
                    },
                    {
                        'id': 'stage_4',
                        'operator': 'airflow.operators.bash_operator.BashOperator',
                        'params': {
                            'bash_command': 'echo "hello world"'
                        }
                    }
                ]
            }
        ]

        prepare_clean_tmp_dir(folder_name='config_cache')
        self.configs_path = os.path.join(tmp_dir_path(), 'config_cache')
        ensure_dir_exists(self.configs_path)

        for config in self.configs:
            config_path = os.path.join(self.configs_path, '{}.yml'.format(config['name']))
            dict_to_yml(config, config_path)

        configs_cache = ConfigCache(configs_path=self.configs_path)
        configs_cache.refresh_cache()

        self.dag_loader = DagLoader(
            project="test_project",
            configs_path=self.configs_path,
            event_handlers=[]
        )

    def test_load(self):
        dags = self.dag_loader.load(global_symbol_table=globals())
        for variable_name, dag in dags:
            self.assertEqual(globals()[variable_name], dag)

    def test_dag_id_prefix(self):
        self.dag_loader.dag_id_prefix = 'prefix_'
        dags = self.dag_loader.load(global_symbol_table=globals())

        self.assertEqual(len([dag for var_name, dag in dags if dag.dag_id == 'pipeline_a_v1']), 0)
        self.assertEqual(len([dag for var_name, dag in dags if dag.dag_id == 'prefix_pipeline_a_v1']), 1)
        self.assertEqual(len([dag for var_name, dag in dags if dag.dag_id == 'pipeline_b_v1']), 0)
        self.assertEqual(len([dag for var_name, dag in dags if dag.dag_id == 'prefix_pipeline_b_v1']), 1)

    def test_load_dag_dependencies(self):
        dags = self.dag_loader.load(global_symbol_table=globals())

        # Pipeline A
        # ==========
        dag = next(dag for var_name, dag in dags if dag.dag_id == 'pipeline_a_v1')

        task_ids = list(map(lambda task: task.task_id, dag.tasks))
        self.assertCountEqual(task_ids, ['start', 'stage_1', 'stage_2', 'stage_2b', 'success'])

        start = dag.get_task('start')
        stage_1 = dag.get_task('stage_1')
        stage_2 = dag.get_task('stage_2')
        stage_2b = dag.get_task('stage_2b')
        success = dag.get_task('success')

        self.assertCountEqual(list(map(lambda task: task.task_id, start.upstream_list)), [])
        self.assertCountEqual(list(map(lambda task: task.task_id, stage_1.upstream_list)), ['start'])
        self.assertCountEqual(list(map(lambda task: task.task_id, stage_2.upstream_list)), ['stage_1'])
        self.assertCountEqual(list(map(lambda task: task.task_id, stage_2b.upstream_list)), ['stage_1'])
        self.assertCountEqual(list(map(lambda task: task.task_id, success.upstream_list)), ['stage_2', 'stage_2b'])

        # Pipeline B
        # ==========
        dag = next(dag for var_name, dag in dags if dag.dag_id == 'pipeline_b_v1')

        task_ids = list(map(lambda task: task.task_id, dag.tasks))
        self.assertCountEqual(task_ids, ['start', 'sensor_1', 'sensor_2', 'stage_1b', 'stage_2c', 'success'])

        start = dag.get_task('start')
        sensor_1 = dag.get_task('sensor_1')
        sensor_2 = dag.get_task('sensor_2')
        stage_1b = dag.get_task('stage_1b')
        stage_2c = dag.get_task('stage_2c')
        success = dag.get_task('success')

        self.assertCountEqual(list(map(lambda task: task.task_id, start.upstream_list)), [])
        self.assertCountEqual(list(map(lambda task: task.task_id, sensor_1.upstream_list)), ['start'])
        self.assertCountEqual(list(map(lambda task: task.task_id, sensor_2.upstream_list)), ['start'])
        self.assertCountEqual(list(map(lambda task: task.task_id, stage_1b.upstream_list)), ['sensor_1', 'sensor_2'])
        self.assertCountEqual(list(map(lambda task: task.task_id, stage_2c.upstream_list)), ['sensor_1', 'sensor_2'])
        self.assertCountEqual(list(map(lambda task: task.task_id, success.upstream_list)), ['stage_1b', 'stage_2c'])

        # Pipeline C
        # ==========
        dag = next(dag for var_name, dag in dags if dag.dag_id == 'pipeline_c_v1')

        task_ids = list(map(lambda task: task.task_id, dag.tasks))
        self.assertCountEqual(task_ids,
                              ['pre_hook_1', 'pre_hook_2', 'start', 'sensor_3', 'stage_3', 'stage_4', 'success'])

        pre_hook_1 = dag.get_task('pre_hook_1')
        pre_hook_2 = dag.get_task('pre_hook_2')
        start = dag.get_task('start')
        sensor_3 = dag.get_task('sensor_3')
        stage_3 = dag.get_task('stage_3')
        stage_4 = dag.get_task('stage_4')
        success = dag.get_task('success')

        self.assertCountEqual(list(map(lambda task: task.task_id, pre_hook_1.upstream_list)), [])
        self.assertCountEqual(list(map(lambda task: task.task_id, pre_hook_2.upstream_list)), ['pre_hook_1'])
        self.assertCountEqual(list(map(lambda task: task.task_id, start.upstream_list)), ['pre_hook_2'])
        self.assertCountEqual(list(map(lambda task: task.task_id, sensor_3.upstream_list)), ['start'])
        self.assertCountEqual(list(map(lambda task: task.task_id, stage_3.upstream_list)), ['sensor_3'])
        self.assertCountEqual(list(map(lambda task: task.task_id, stage_4.upstream_list)), ['sensor_3'])
        self.assertCountEqual(list(map(lambda task: task.task_id, success.upstream_list)), ['stage_3', 'stage_4'])

    def test_load_dag_with_default_dag_params(self):
        dags = self.dag_loader.load(global_symbol_table=globals())

        dag = next(dag for var_name, dag in dags if dag.dag_id == 'pipeline_a_v1')
        self.assertEqual(dag.default_args['foo'], 'bar')

    def test_build_task(self):
        task = DagLoader.build_task(task_id='some_task__id',
                                    dag=DAG(dag_id='some_dag', default_args={'start_date': datetime.datetime.now()}),
                                    operator='airflow.operators.bash_operator.BashOperator',
                                    bash_command='echo "hello world"')

        self.assertIsInstance(task, BashOperator)
        self.assertEqual(task.task_id, 'some_task__id')
        self.assertEqual(task.bash_command, 'echo "hello world"')

    def test_build_task_with_expected_class(self):
        task = DagLoader.build_task(task_id='some_task__id',
                                    dag=DAG(dag_id='some_dag', default_args={'start_date': datetime.datetime.now()}),
                                    operator='airflow.operators.bash_operator.BashOperator',
                                    expected_class=BashOperator,
                                    bash_command='echo "hello world"')
        self.assertIsInstance(task, BashOperator)

        task = DagLoader.build_task(task_id='some_task__id',
                                    dag=DAG(dag_id='some_dag', default_args={'start_date': datetime.datetime.now()}),
                                    operator='airflow.operators.bash_operator.BashOperator',
                                    expected_class=BaseOperator,
                                    bash_command='echo "hello world"')
        self.assertIsInstance(task, BashOperator)

        with self.assertRaises(Exception):
            task = DagLoader.build_task(task_id='some_task__id',
                                        dag=DAG(dag_id='some_dag',
                                                default_args={'start_date': datetime.datetime.now()}),
                                        operator='airflow.operators.bash_operator.BashOperator',
                                        expected_class=PythonOperator,
                                        bash_command='echo "hello world"')


class TestTasksSubDag(TestCase):

    def setUp(self):
        self.task_configs = [
            {
                'id': 'stage_1',
                'operator': 'airflow.operators.bash_operator.BashOperator',
                'params': {
                    'bash_command': 'echo "hello world"'
                }
            },
            {
                'id': 'stage_2',
                'operator': 'airflow.operators.bash_operator.BashOperator',
                'params': {
                    'bash_command': 'echo "hello universe"'
                },
                'upstream': ['stage_1']
            },
            {
                'id': 'stage_2b',
                'operator': 'airflow.operators.bash_operator.BashOperator',
                'params': {
                    'bash_command': 'echo "hello universe in parallel"'
                },
                'upstream': ['stage_1']
            }
        ]

        self.dag = DAG(dag_id='some_dag', default_args={'start_date': datetime.datetime.now()})

        self.sub_dag = TasksSubDag(
            task_configs=self.task_configs,
            dag=self.dag,
            job_name='some_job_name',
            params={}
        )

    def test_leaves(self):
        leaf_ids = [leaf.task_id for leaf in self.sub_dag.leaves]

        self.assertCountEqual(leaf_ids, ['stage_2', 'stage_2b'])

    def test_roots(self):
        root_ids = [root.task_id for root in self.sub_dag.roots]

        self.assertCountEqual(root_ids, ['stage_1'])


class TestDependencySensors(TestCase):

    def setUp(self):
        self.sensor_configs = [
            {
                'id': 'sensor_1',
                'operator': 'airflow.sensors.time_sensor.TimeSensor',
                'params': {
                    'target_time': datetime.time(0, 0, 0)
                }
            },
            {
                'id': 'sensor_2',
                'operator': 'airflow.sensors.time_sensor.TimeSensor',
                'params': {
                    'target_time': datetime.time(0, 0, 0)
                }
            },
            {
                'id': 'sensor_3',
                'operator': 'airflow.sensors.time_sensor.TimeSensor',
                'params': {
                    'target_time': datetime.time(0, 0, 0)
                }
            }
        ]

        self.dependent_sensor_configs = [
            {
                'id': 'sensor_1',
                'operator': 'airflow.sensors.time_sensor.TimeSensor',
                'params': {
                    'target_time': datetime.time(0, 0, 0)
                }
            },
            {
                'id': 'sensor_2',
                'operator': 'airflow.sensors.time_sensor.TimeSensor',
                'params': {
                    'target_time': datetime.time(0, 0, 0)
                },
                'upstream': ['sensor_1']
            },
            {
                'id': 'sensor_3',
                'operator': 'airflow.sensors.time_sensor.TimeSensor',
                'params': {
                    'target_time': datetime.time(0, 0, 0)
                },
                'upstream': ['sensor_1']
            }
        ]

        self.dag = DAG(dag_id='some_dag', default_args={'start_date': datetime.datetime.now()})

        self.dependency_sensors = DependencySensors(
            task_configs=self.sensor_configs,
            dag=self.dag,
            params={}
        )

        self.dependent_dependency_sensors = DependencySensors(
            task_configs=self.dependent_sensor_configs,
            dag=self.dag,
            params={}
        )

    def test_leaves(self):
        task_ids = [task.task_id for task in self.dependency_sensors.leaves]
        self.assertCountEqual(task_ids, ['sensor_1', 'sensor_2', 'sensor_3'])

    def test_roots(self):
        task_ids = [task.task_id for task in self.dependency_sensors.roots]
        self.assertCountEqual(task_ids, ['sensor_1', 'sensor_2', 'sensor_3'])

    def test_dependent_leaves(self):
        task_ids = [task.task_id for task in self.dependent_dependency_sensors.leaves]
        self.assertCountEqual(task_ids, ['sensor_2', 'sensor_3'])

    def test_dependent_roots(self):
        task_ids = [task.task_id for task in self.dependent_dependency_sensors.roots]
        self.assertCountEqual(task_ids, ['sensor_1'])

    def test_timeout_set_to_zero(self):
        for sensor in self.dependency_sensors.tasks.values():
            self.assertEqual(sensor.timeout, 0)



import datetime
from unittest import TestCase
from unittest.mock import MagicMock, patch, call

import pytz
from airflow import DAG
from airflow.models import TaskInstance

from awehflow.operators.flow import EventEmittingOperator, StartOperator, SuccessOperator, FailureOperator


class TestEventEmittingOperatorOperator(TestCase):
    def setUp(self):
        self.dag = DAG(dag_id='some_dag', start_date=datetime.datetime.now())
        self.handler1 = MagicMock()
        self.handler2 = MagicMock()
        self.task = EventEmittingOperator(
            dag=self.dag,
            task_id='task',
            event_handlers=[self.handler1, self.handler2]
        )

        self.task_instance = TaskInstance(task=self.task, execution_date=datetime.datetime(1985, 3, 29))
        self.task_instance_context = self.task_instance.get_template_context()

    def test_execute(self):
        with self.assertRaises(Exception):
            self.task.execute(self.task_instance_context)

    def test_emit_event(self):
        self.task.emit_event('some_event_name', body={'field': 'val'})
        self.handler1.handle.assert_called_once_with({
            'name': 'some_event_name',
            'body': {'field': 'val'}
        })
        self.handler2.handle.assert_called_once_with({
            'name': 'some_event_name',
            'body': {'field': 'val'}
        })


class TestStartOperator(TestCase):
    def setUp(self):
        self.dag = DAG(dag_id='some_dag', start_date=datetime.datetime.now())
        self.task = StartOperator(
            dag=self.dag,
            task_id='task',
            project='some_project',
            event_handlers=[],
            job_name='some_job_name',
            engineers=[{'name': 'Batman'}]
        )

        self.task_instance = TaskInstance(task=self.task, execution_date=datetime.datetime(1985, 3, 29, tzinfo=pytz.UTC))
        self.task_instance_context = self.task_instance.get_template_context()

    @patch('awehflow.operators.flow.utc_now')
    def test_execute(self, mock_utc_now):
        self.task.emit_event = MagicMock()
        mock_utc_now.return_value = datetime.datetime(1985, 3, 29)
        self.task_instance_context['dag_run'] = MagicMock()
        self.task_instance_context['dag_run'].run_id = 'some_dag_run_id'
        self.task.execute(self.task_instance_context)

        self.task.emit_event.assert_called_once_with('start', {
            'run_id': 'some_dag_run_id',
            'dag_id': 'some_dag',
            'name': 'some_job_name',
            'project': 'some_project',
            'engineers': [{'name': 'Batman'}],
            'status': 'running',
            'start_time': datetime.datetime(1985, 3, 29),
            'reference_time': datetime.datetime(1985, 3, 30, tzinfo=pytz.UTC)
        })


class TestSuccessOperator(TestCase):
    def setUp(self):
        self.dag = DAG(dag_id='some_dag', start_date=datetime.datetime.now())
        self.task = SuccessOperator(
            dag=self.dag,
            task_id='task',
            project='some_project',
            event_handlers=[],
            job_name='some_job_name',
            engineers=[{'name': 'Batman'}]
        )

        self.task_instance = TaskInstance(task=self.task, execution_date=datetime.datetime(1985, 3, 29))
        self.task_instance_context = self.task_instance.get_template_context()

    @patch('awehflow.operators.flow.utc_now')
    def test_execute(self, mock_utc_now):
        self.task.emit_event = MagicMock()
        mock_utc_now.return_value = datetime.datetime(1985, 3, 29)
        self.task_instance_context['dag_run'] = MagicMock()
        self.task_instance_context['dag_run'].run_id = 'some_dag_run_id'
        self.task.execute(self.task_instance_context)

        self.task.emit_event.assert_called_once_with('success', {
            'run_id': 'some_dag_run_id',
            'dag_id': 'some_dag',
            'name': 'some_job_name',
            'project': 'some_project',
            'engineers': [{'name': 'Batman'}],
            'status': 'success',
            'end_time': datetime.datetime(1985, 3, 29)
        })


class TestFailureOperator(TestCase):
    def setUp(self):
        self.dag = DAG(dag_id='some_dag', start_date=datetime.datetime.now())
        self.task = FailureOperator(
            dag=self.dag,
            task_id='task',
            project='some_project',
            event_handlers=[],
            job_name='some_job_name',
            engineers=[{'name': 'Batman'}]
        )

        self.task_instance = TaskInstance(task=self.task, execution_date=datetime.datetime(1985, 3, 29))
        self.task_instance_context = self.task_instance.get_template_context()

    @patch('awehflow.operators.flow.utc_now')
    def test_execute(self, mock_utc_now):
        self.task.emit_event = MagicMock()
        mock_utc_now.return_value = datetime.datetime(1985, 3, 29)
        self.task_instance_context['dag_run'] = MagicMock()
        self.task_instance_context['dag_run'].run_id = 'some_dag_run_id'
        self.task_instance_context['task_instance'] = MagicMock()
        self.task_instance_context['task_instance'].log_url = 'example.com'
        self.task_instance_context['task_instance'].task_id = 'task'
        self.task_instance_context['exception'] = 'Exception message'
        self.task.execute(self.task_instance_context)

        self.task.emit_event.assert_called_once_with('failure', {
            'run_id': 'some_dag_run_id',
            'dag_id': 'some_dag',
            'name': 'some_job_name',
            'project': 'some_project',
            'engineers': [{'name': 'Batman'}],
            'status': 'failure',
            'error': {
                'task_id': 'task',
                'message': 'Exception message',
                'log_url': 'example.com'
            },
            'end_time': datetime.datetime(1985, 3, 29)
        })



from unittest import TestCase
from unittest.mock import MagicMock

from awehflow.events.base import EventHandler


class TestEventHandler(TestCase):
    def test_handle(self):
        handler = EventHandler()
        handler.catch_all = MagicMock()

        event = {'name': 'start'}
        handler.handle(event)
        handler.catch_all.assert_called_once_with(event)

    def test_handle_with_custom_event_method(self):
        class DummyEventHandler(EventHandler):
            def boogie(self, event):
                pass

        handler = DummyEventHandler()
        handler.boogie = MagicMock()
        handler.catch_all = MagicMock()

        event = {'name': 'boogie'}
        handler.handle(event)
        handler.boogie.assert_called_once_with(event)
        handler.catch_all.assert_not_called()


from awehflow.events.base import EventHandler


class AlertsEventHandler(EventHandler):
    def __init__(self, alerters, alert_on):
        self.alerters = alerters
        self.alert_on = alert_on

    def catch_all(self, event):
        if event.get('name') in self.alert_on:
            for alerter in self.alerters:
                alerter.alert(event)



import os
import shutil

import yaml

THIS_DIR = os.path.dirname(os.path.abspath(__file__))


def tmp_dir_path():
    return os.path.join(THIS_DIR, '..', 'tmp')


def remove_dir(path):
    shutil.rmtree(path)


def ensure_dir_exists(path):
    if not os.path.exists(path):
        os.makedirs(path)


def prepare_clean_tmp_dir(folder_name=None):
    if not folder_name and os.path.exists(tmp_dir_path()):
        remove_dir(tmp_dir_path())

    ensure_dir_exists(tmp_dir_path())

    if folder_name:
        folder_path = os.path.join(tmp_dir_path(), folder_name)
        if os.path.exists(folder_path):
            remove_dir(folder_path)
        os.makedirs(folder_path)


def dict_to_yml(input_dict, path):
    with open(path, 'w') as fp:
        yaml.dump(input_dict, fp)
